@@ -15,7 +15,7 @@ from pydantic import BaseModel, Field
 from tqdm import tqdm
 from wonderwords import RandomWord
 
-from .cli import get_api_credentials
+from .cli import get_api_credentials, refresh_bearer_token, validate_api_http_response
 import sys
 # Uploads send several requests to allow for threadpool refreshing.
 # Threadpool hogs memory and new ones need to be created.
@@ -65,6 +65,7 @@ class AtlasClient:
 
         '''
 
+        refresh_bearer_token()
         self.credentials = get_api_credentials()
 
         if self.credentials['tenant'] == 'staging':
@@ -83,6 +84,7 @@ class AtlasClient:
                 self.atlas_api_path + "/v1/user",
                 headers=self.header,
             )
+            response = validate_api_http_response(response)
             if not response.status_code == 200:
                 logger.info("Your authorization token is no longer valid.")
         else:
@@ -95,6 +97,7 @@ class AtlasClient:
             self.atlas_api_path + "/v1/user",
             headers=self.header,
         )
+        response = validate_api_http_response(response)
         if not response.status_code == 200:
             raise ValueError("Your authorization token is no longer valid. Run `nomic login` to obtain a new one.")
 
@@ -265,7 +268,7 @@ class AtlasClient:
 
         return response.json()
 
-    def _validate_user_supplied_metadata(self, data: List[Dict], project):
+    def _validate_user_supplied_metadata(self, data: List[Dict], project, replace_empty_string_values_with_string_null=True):
         '''
         Validates the users metadata for Atlas compatability.
         If unique_id_field is specified, validates that each datum has that field. If not, adds it
@@ -273,6 +276,7 @@ class AtlasClient:
         Args:
             data: the user supplied list of data dictionaries
             project: the atlas project you are validating the data for.
+            replace_empty_string_values_with_string_null: replaces empty string values with string nulls in all datums
 
         Returns:
 
@@ -308,8 +312,11 @@ class AtlasClient:
 
                 if project['modality'] == 'text':
                     if isinstance(datum[key], str) and len(datum[key]) == 0:
-                        msg = 'Datum {} had an empty string for key: {}'.format(datum, key)
-                        raise ValueError(msg)
+                        if replace_empty_string_values_with_string_null:
+                            datum[key] = 'null'
+                        else:
+                            msg = 'Datum {} had an empty string for key: {}'.format(datum, key)
+                            raise ValueError(msg)
 
                 if not isinstance(datum[key], (str, float, int)):
                     raise Exception(f"Metadata sent to Atlas must be a flat dictionary. Values must be strings, floats or ints. Key `{key}` of datum {str(datum)} is in violation.")
@@ -318,10 +325,27 @@ class AtlasClient:
             logger.info(f"A datum you supplied lacked the unique id field `{unique_id_field}`. Added it for you.")
 
 
+    def is_project_accepting_data(self, project_id: str):
+        '''
+        Checks if the project can accept data. Projects cannot accept data when they are being indexed.
+
+        **Parameters:**
+
+        * **project_id** - The id of the project you are checking.
+
+        **Returns:** True if project is unlocked for data additions, false otherwise.
+        '''
+        response = requests.get(
+            self.atlas_api_path+ f"/v1/project/{project_id}",
+            headers=self.header,
+        )
+        project = response.json()
+
+        return not project['insert_update_delete_lock']
 
 
     def add_embeddings(
-        self, project_id: str, embeddings: np.array, data: List[Dict], shard_size=1000, num_workers=10, pbar=None
+        self, project_id: str, embeddings: np.array, data: List[Dict], shard_size=1000, num_workers=10, replace_empty_string_values_with_string_null=True, pbar=None
     ):
         '''
         Adds embeddings to an embedding project. Pair each embedding with meta-data to explore your embeddings.
@@ -333,6 +357,7 @@ class AtlasClient:
         * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
         * **shard_size** - Embeddings are uploaded in parallel by many threads. Adjust the number of embeddings to upload by each worker.
         * **num_workers** - The number of worker threads to upload embeddings with.
+        * **replace_empty_string_values_with_string_null** - Replaces empty values in metadata with null. If false, will fail if empty values are supplied.
 
         **Returns:** True on success.
         '''
@@ -351,9 +376,12 @@ class AtlasClient:
             msg = 'Cannot add embedding to project with modality: {}'.format(project['modality'])
             raise ValueError(msg)
 
+        if project['insert_update_delete_lock']:
+            raise Exception("Project is currently indexing and cannot ingest new datums. Try again later.")
+
         progressive = len(project['atlas_indices']) > 0
         try:
-            self._validate_user_supplied_metadata(data=data, project=project)
+            self._validate_user_supplied_metadata(data=data, project=project, replace_empty_string_values_with_string_null=replace_empty_string_values_with_string_null)
         except BaseException as e:
             raise e
 
@@ -365,6 +393,7 @@ class AtlasClient:
         # Actually do the upload
         def send_request(i):
             data_shard = data[i : i + shard_size]
+
             if get_object_size_in_bytes(data_shard) > 8000000:
                 raise Exception("Your metadata upload shards are to large. Try decreasing the shard size or removing un-needed fields from the metadata.")
             self._ensure_metadata(data_shard)
@@ -396,6 +425,8 @@ class AtlasClient:
                         logger.error(f"Shard upload failed: {response.json()}")
                         if 'more datums exceeds your organization limit' in response.json():
                             return False
+                        if 'Project transaction lock is held':
+                            raise Exception("Project is currently indexing and cannot ingest new datums. Try again later.")
                     except requests.exceptions.JSONDecodeError:
                         if response.status_code == 413:
                             logger.error("Shard upload failed: you are sending meta-data data is to large.")
@@ -469,7 +500,7 @@ class AtlasClient:
 
             hyperparameters = {
                 'dataset_buffer_size': 1000,
-                'batch_size': 4,
+                'batch_size': 20,
                 'polymerize_by': 'charchunk',
             }
             text_build_template = {
@@ -525,7 +556,7 @@ class AtlasClient:
         to_return['project_id'] = project['id']
         return CreateIndexResponse(**to_return)
 
-    def add_text(self, project_id: str, data: List[Dict], shard_size=1000, num_workers=10, pbar=None):
+    def add_text(self, project_id: str, data: List[Dict], shard_size=1000, num_workers=10, replace_empty_string_values_with_string_null=True, pbar=None):
         '''
         Adds data to a text project.
 
@@ -535,6 +566,7 @@ class AtlasClient:
         * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
         * **shard_size** - Embeddings are uploaded in parallel by many threads. Adjust the number of embeddings to upload by each worker.
         * **num_workers** - The number of worker threads to upload embeddings with.
+        * **replace_empty_string_values_with_string_null** - Replaces empty values in metadata with null. If false, will fail if empty values are supplied.
 
         **Returns:** True on success.
         '''
@@ -548,14 +580,20 @@ class AtlasClient:
             self.atlas_api_path+ f"/v1/project/{project_id}",
             headers=self.header,
         )
+
         project = response.json()
         if project['modality'] != 'text':
             msg = 'Cannot add text to project with modality: {}'.format(project['modality'])
             raise ValueError(msg)
 
         progressive = len(project['atlas_indices']) > 0
+
+        if project['insert_update_delete_lock']:
+            raise Exception("Project is currently indexing and cannot ingest new datums. Try again later.")
+
         try:
-            self._validate_user_supplied_metadata(data=data, project=project)
+            self._validate_user_supplied_metadata(data=data, project=project,
+                                                  replace_empty_string_values_with_string_null=replace_empty_string_values_with_string_null)
         except BaseException as e:
             raise e
 
@@ -595,6 +633,8 @@ class AtlasClient:
                         logger.error(f"Shard upload failed: {response.json()}")
                         if 'more datums exceeds your organization limit' in response.json():
                             return False
+                        if 'Project transaction lock is held':
+                            raise Exception("Project is currently indexing and cannot ingest new datums. Try again later.")
                     except requests.exceptions.JSONDecodeError:
                         logger.error(f"Shard upload failed: {response}")
                         continue
@@ -610,7 +650,7 @@ class AtlasClient:
             if failed:
                 logger.warning("Text upload partially succeeded.")
             else:
-                logger.warning("Text upload succeeded.")
+                logger.info("Text upload succeeded.")
 
         return True
 
@@ -654,6 +694,7 @@ class AtlasClient:
 
         if map_name:
             project_name = map_name
+            index_name = map_name
         if map_description:
             description = map_description
 
@@ -765,6 +806,19 @@ class AtlasClient:
 
         #Update maps
         # finally, update all the indices
+        return self.refresh_maps(project_id=project_id)
+
+    def refresh_maps(self, project_id: str):
+        '''
+        Refresh all maps in a project with the latest state.
+
+        **Parameters:**
+
+        * **project_id** - The id of the project whose maps will be refreshed.
+
+        **Returns:** a list of jobs
+        '''
+
         response = requests.post(
             self.atlas_api_path + "/v1/project/update_indices",
             headers=self.header,
@@ -777,6 +831,7 @@ class AtlasClient:
 
 
 
+
     def map_text(
         self,
         data: List[Dict],
@@ -817,6 +872,7 @@ class AtlasClient:
 
         if map_name:
             project_name = map_name
+            index_name = map_name
         if map_description:
             description = map_description
 
@@ -835,8 +891,6 @@ class AtlasClient:
         )
 
         shard_size = 1000
-        if len(data) > 10000:
-            shard_size = 2500
 
         logger.info("Uploading text to Nomic's neural database Atlas.")
 
